# -*- coding: utf-8 -*-
from sqlite3 import dbapi2 as sqlite
import sqlite3
import pubsub
import blamobjects
import logging
import os

logger = logging.getLogger(__name__)

class BlamStore(object):
    
    def __init__(self, path, autoSave = True):
        self.Path = path
        self.AutoSave = autoSave
        self.Dirty = False
        self.Version = 1
        existing = True
        #print "mach was!"
        if not os.path.exists(self.Path):
            #print "mach ich ja"
            logger.debug('Initializing: '+ self.Path)
            connection = self.Initialize()
            existing = False
        else:
            logger.debug('Loading: '+ self.Path)
            connection = sqlite.connect(self.Path)
        self.dbconn = connection
        self.Dirty = True
        self.commitIfAppropriate()
        self.Meta = self.GetMeta()
        if self.Meta['VERSION'] < self.Version:
            self.upgradeDB(self.Meta['VERSION'])
        self.Subscriptions = (
            (self.onTagAdded, "Tag.added"),
            (self.onTagRemoved, "Tag.removed"),
            (self.removeTagComicLinks, "Tag.removed"),
            (self.onStripNumber, "Strip.updated.number"),
            (self.onStripComic, "Strip.updated.comic"),
            (self.onImagePath, "Image.updated.path"),
            (self.onImageStrip, "Image.updated.strip"),
            (self.onExit, "exiting"),
        )
        for callback, topic in self.Subscriptions:
            pubsub.subscribe(topic, callback)
        self.blamModel = self.CreateModel()
            
    def getModel(self):
        return self.blamModel

    def CreateModel(self):
        logger.debug('Creating model...')
        return blamobjects.blammodel(self)
        
    def CreateComic(self, name):
        cursor = self.dbconn.cursor()
        cursor.execute('INSERT INTO comics VALUES (null, ?)', (name,))
        ID = cursor.lastrowid
        self.Dirty = True
        self.commitIfAppropriate()
        comic = blamobjects.Comic(self,ID,name)
        return comic
        
    def CreateTag(self, name):
        cursor = self.dbconn.cursor()
        cursor.execute('INSERT INTO tags VALUES (null, ?)', (name,))
        ID = cursor.lastrowid
        self.Dirty = True
        self.commitIfAppropriate()
        tag = blamobjects.Tag(self,ID,name)
        return tag
        
    def CreateStrip(self, name):
        cursor = self.dbconn.cursor()
        cursor.execute('INSERT INTO strips VALUES (null, null, null, ?)', (name,))
        ID = cursor.lastrowid
        self.Dirty = True
        self.commitIfAppropriate()
        strip = blamobjects.Strip(self,ID,name)
        return strip
        
    def CreateImage(self, name):
        cursor = self.dbconn.cursor()
        cursor.execute('INSERT INTO images VALUES (null, null, null, ?)', (name,))
        ID = cursor.lastrowid
        self.Dirty = True
        self.commitIfAppropriate()
        image = blamobjects.Image(self,ID,name)
        return image
        
    def Initialize(self):
        connection = sqlite.connect(self.Path)
        cursor = connection.cursor()
        cursor.execute('CREATE TABLE comics (id INTEGER PRIMARY KEY, name VARCHAR(255))')
        cursor.execute('CREATE TABLE tags (id INTEGER PRIMARY KEY, name VARCHAR(255))')
        cursor.execute('CREATE TABLE strips (id INTEGER PRIMARY KEY, comicId INTEGER, number FLOAT, name VARCHAR(255))')
        cursor.execute('CREATE TABLE images (id INTEGER PRIMARY KEY, stripId INTEGER, path VARCHAR(255), name VARCHAR(255))')
        cursor.execute('CREATE TABLE tags_comics_link (id INTEGER PRIMARY KEY, comicId INTEGER, tagId INTEGER)')
        cursor.execute('CREATE TABLE meta (id INTEGER PRIMARY KEY, name VARCHAR(255), value VARCHAR(255))')
        cursor.execute('INSERT INTO meta VALUES (null, ?, ?)', ('VERSION', self.Version))
        return connection
        
    def GetMeta(self):
        try:
            results = self.dbconn.cursor().execute('SELECT * FROM meta').fetchall()
        except sqlite3.OperationalError:
            meta = {'VERSION': 0}
        else:
            meta = {}
            for uid, key, value in results:
                # All values come in as strings but some we want to cast.
                if key == "VERSION":
                    value = int(value)
                meta[key] = value
        return meta
        
    def upgradeDB(self, oldVersion):
        # Serious stuff to do here, for now:
        logger.error( "HAT HIER WIEDER JEMAND VERGESSEN DIE DB ZU LÖSCHEN??")
            
    def _get(self, create, table):
        return [create(result) for result in self.dbconn.cursor().execute("SELECT * FROM " + table).fetchall()] 
        
    def getComics(self):
       return self._get(self.result2comic, "comics")
       
    def getTags(self):
       return self._get(self.result2tag, "tags")
       
    def getStrips(self):
       return self._get(self.result2strip, "strips")
       
    def getImages(self):
       return self._get(self.result2image, "images")
       
    def result2comic(self, result):
        ID, name = result
        comic = blamobjects.Comic(self, ID, name)
        return comic
        
    def result2tag(self, result):
        ID, name = result
        return blamobjects.Tag(self, ID, name)
        
    def result2strip(self, result):
        ID, comicID, number, name = result
        strip = blamobjects.Strip(self, ID, name)
        if number:
            strip._number = number
        return strip
        
    def result2image(self, result):
        ID, stripID, path, name = result
        image = blamobjects.Image(self, ID, name)
        if path:
            image._path = path
        return image
        
    def onStripNumber(self, number, strip):
        logger.debug("Setting number of " +str(strip) + " to " + str(number))
        self.dbconn.cursor().execute('UPDATE strips SET number=? WHERE id=?', (number, strip.ID))
        self.Dirty = True
        self.commitIfAppropriate()
        
    def onStripComic(self, comic, strip):
        logger.debug("Setting Comic of " +str(strip) + " to " + str(comic))
        self.dbconn.cursor().execute('UPDATE strips SET comicId=? WHERE id=?', (comic.ID, strip.ID))
        self.Dirty = True
        self.commitIfAppropriate()
        
    def onTagAdded(self, tag, comic):
        logger.debug("Adding " + str(tag) + " to " + str(comic))
        cursor = self.dbconn.cursor()
        cursor.execute('INSERT INTO tags_comics_link VALUES (null, ?, ?)', (comic.ID, tag.ID,))
        self.Dirty = True
        self.commitIfAppropriate()
        
    def onTagRemoved(self, tag):
        logger.debug("Removing " + str(tag))
        cursor = self.dbconn.cursor()
        cursor.execute('DELETE FROM tags WHERE id=?', (tag.ID,))
        self.Dirty = True
        self.commitIfAppropriate()
        
    def removeTagComicLinks(self, tag):
        logger.debug("Removing Links of Tag " + str(tag))
        cursor = self.dbconn.cursor()
        cursor.execute('DELETE FROM tags_comics_link WHERE tagId=?', (tag.ID,))
        self.Dirty = True
        self.commitIfAppropriate()
        
    def onImagePath(self, path, image):
        logger.debug("Setting Path of " + str(image) + " to " + str(path))
        cursor = self.dbconn.cursor()
        cursor.execute('UPDATE images SET path=? WHERE id=?', (path, image.ID))
        self.Dirty = True
        self.commitIfAppropriate()
        
    def onImageStrip(self, strip, image):
        logger.debug("Setting Strip of " + str(image) + " to " + str(strip))
        if not image in strip._images:
            strip._images.append(image)
        cursor = self.dbconn.cursor()
        cursor.execute('UPDATE images SET stripId=? WHERE id=?', (strip.ID, image.ID))
        self.Dirty = True
        self.commitIfAppropriate()
        
    def onExit(self):
        logging.debug("Exiting")
        
    def dressObjects(self, comics, tags, strips, images):
        taglinks = self.dbconn.cursor().execute("SELECT * FROM tags_comics_link").fetchall()
        for id, comicID, tagID in taglinks:
            comic = comics.getById(comicID)
            tag = tags.getById(tagID)
            comic._tags.append(tag)
            tag._comics.append(comic)
            logging.debug("Linked " + str(comic) + " to " + str(tag))
        striplinks = self.dbconn.cursor().execute("SELECT * FROM strips").fetchall()
        for id, comicID, number, name in striplinks:
            if comicID:
                comic = comics.getById(comicID)
                strip = strips.getById(id)
                comic._strips.append(strip)
                strip._comic = comic
                logging.debug("Linked " + str(comic) + " to " + str(strip))
        imagelinks = self.dbconn.cursor().execute("SELECT * FROM images").fetchall()
        for ID, stripID, path, name in imagelinks:
            if stripID:
                image = images.getById(ID)
                strip = strips.getById(stripID)
                image._strip = strip
                strip._images.append(image)
                logging.debug("Linked " + str(image) + " to " + str(strip))
        
        
    def Save(self):
        import time; t = time.time()
        self.dbconn.commit()
        logger.debug("Committed in %s seconds" % (time.time()-t))
        self.Dirty = False
        
    def Close(self):
        self.dbconn.close()
        for callback, topic in self.Subscriptions:
            unsubscribe(topic, callback)
    
    def commitIfAppropriate(self):
        if self.AutoSave and self.Dirty:
            self.Save()
            
    def onExit(self, message):
        if self.Dirty:
            pubsub.publish("warning.dirty exit", message.data)

    def __del__(self):
        self.commitIfAppropriate()
        self.Close()
