"""This module contains a function to download things from http as a string"""
import urllib2
import StringIO
import gzip
import logging
import re

logger = logging.getLogger(__name__)

def apply_regex(string, regex):
    """applies the regex on the string"""
    #print regex
    #r = re.compile(regex, re.MULTILINE)
    results = re.findall(regex, string, re.MULTILINE)
    if results is not None:
        return results
    else:
        logger.debug('Regex ' + regex + ' not found.')
        return []

def download_html(url):
    """url: the url to download"""
    #print url
    request = urllib2.Request(url)
    request.add_header('Accept-encoding', 'gzip')
    response = None
    try:
        response = urllib2.urlopen(request)
    except Exception, e:
        logger.error(e)
        raise e
    html = response.read()
    if response.headers.dict.has_key("content-encoding") and response.headers.dict["content-encoding"] == "gzip":
        gzipper = gzip.GzipFile(fileobj = StringIO.StringIO(html))
        html = gzipper.read()
    return html


def download_image(url):
    """url: the image to download"""
    #print "Image: ",url
    try:
        request = urllib2.Request(url)
        request.add_header('Referer', url)
        response = urllib2.urlopen(request)
    except Exception, e:
        logging.getLogger().error("Error while downloadind image: " + str(url) +" "+ str(e))
        return None
    return response

def get_urls_for_comic(comic):
    baseURL = comic.get('baseURL')
    #print "BASE", baseURL
    if not baseURL:
        logger.debug('Comic ' + comic.get('name') + ' does not have a baseURL')
        return {}
    html = download_html(baseURL)
    #print html
    regexes = {}
    for ex in ['imageRegex','nextRegex','prevRegex','titleRegex', 'numberRegex']:
        current = comic.get(ex)
        if current:
            regexes[ex] = apply_regex(html, current)
        else:
            logger.debug('Comic ' + comic.get('name') + ' does not have a regex ' + ex)
    #print regexes
    return regexes
