
import gtk, string, gobject, logging
import config, favicons, pubsub, blamobjects, context_menus, pubsub
import session


logger = logging.getLogger(__name__)
iter_cache = {}


class Tree(gtk.TreeView):
    def __init__(self):
        gtk.TreeView.__init__(self)
        
    def remove(self, iter):
        self.get_model().remove(iter)

    def insert(self, item):
        return None

    def clear(self):
        self.get_model().clear()
    
    def on_treeview_cursor_changed(self, widget):
        self.selected_item = None
        #Get the current selection in the gtk.TreeView
        selection = widget.get_selection()
        # Get the selection iter
        model, selection_iter = selection.get_selected()
        if (selection_iter and model):
            #Something is selected so get the object
            obj = model.get_value(selection_iter, 0)
            self.selected_item = [obj, model, selection_iter] 


class Category(object):
    def __init__(self, name):
        self.name = name


class MainTree(Tree):
    
    def __init__(self):
        """
        @params tree - treeview
        """
        Tree.__init__(self)
        #obj, name
        self.set_model(gtk.TreeStore(gobject.TYPE_PYOBJECT,gtk.gdk.Pixbuf, str))
        self.set_headers_visible(False)
             
        column = gtk.TreeViewColumn()
        # Icon Renderer
        renderer = gtk.CellRendererPixbuf()
        column.pack_start(renderer, expand = False)
        column.add_attribute(renderer, "pixbuf", 1)
        # Text Renderer
        renderer = gtk.CellRendererText()
        column.pack_start(renderer, expand = True)
        column.add_attribute(renderer, "markup", 2)

        self.append_column(column)
        self.set_search_column(2)
        column.set_sort_column_id(2)
        self.insert_categories()
        self.iters = {}
        self.selected_item = None
        pubsub.subscribe("Tag.Removed", self.delete_tag)
        pubsub.subscribe("Tag.created", self.insert_tag)
        
        self.connect('cursor_changed', self.on_cursor_changed)
        self.connect('button-press-event', self.on_button_press_event)
        

    def insert_categories(self):
        self.all_iter = self.get_model().append(None, [Category('all'),None,"<b>All Comics</b>"])

    def insert_tag(self, name, tag):
        #print type(tag)
        #print dir(tag)
        #print tag.toString()
        self.iters[tag] = self.get_model().append(self.all_iter, [tag, None, tag.Name])
        
    def delete_tag(self, tag):
        self.remove(self.iters[tag])
        del self.iters[tag]

    def on_cursor_changed(self, widget):
        self.on_treeview_cursor_changed(widget)
        if self.selected_item != None:
            obj, model, selection_iter = self.selected_item
            s = session.get_session()
            s['comicTree'].clear()
            #tag selected
            if isinstance(obj, blamobjects.Tag):
                self.load_comics(obj)
            if isinstance(obj, Category):
                if obj.name == 'all':
                    self.load_all_comics()
    

    def load_comics(self, tag):
        s = session.get_session()
        for c in s['model'].getComicsForTag(tag):
            s['comicTree'].insert_comic(c)
    
    def load_all_comics(self):
        s = session.get_session()
        for c in s['model'].Comics:
            s['comicTree'].insert_comic(c)
            s['comicTree'].insert_comic(c)  
                

    def on_button_press_event(self, widget, event):
        #right-click
        if event.button == 3:
            if self.selected_item:
                obj, model, selection_iter = self.selected_item
                contextmenu = context_menus.MaintreeContextMenu(obj)
            else:
                contextmenu = context_menus.MaintreeContextMenu()
            contextmenu.show(event)



class ComicTree(Tree):
    def __init__(self):
        """
        @params tree - treeview
        """
        Tree.__init__(self)
        
        #obj, name
        self.set_model(gtk.TreeStore(gobject.TYPE_PYOBJECT,gtk.gdk.Pixbuf, str))
             
        ## Icon 
        column = gtk.TreeViewColumn('Icon', gtk.CellRendererPixbuf(), pixbuf = 1)
        self.append_column(column)

        ## Text Renderer
        column = gtk.TreeViewColumn('Name', gtk.CellRendererText(), markup = 2)
        self.append_column(column)

        self.set_search_column(2)
        column.set_sort_column_id(2)
        self.set_headers_visible(True)
        
        
        
        self.connect('cursor_changed', self.on_cursor_changed)
        self.connect('button-press-event', self.on_button_press_event)

        self.selected_comic = None

    def insert_comic(self, comic):
        try:
            favicon = gtk.gdk.pixbuf_new_from_file(favicons.get_favicon(comic)).scale_simple(40,40, gtk.gdk.INTERP_BILINEAR)
        except Exception, e:
            logger.error(str(e))
            return
        iter_cache[comic] = self.get_model().append(None, [comic, favicon, comic.Name])

    
    
    #_______________________SIGNAL HANDLERS__________________________________
    
    def on_button_press_event(self, widget, event):
        if event.button == 3:
            if self.selected_comic:
                contextmenu = context_menus.ComicContextMenu(self.selected_comic)
                contextmenu.show(event)
    
    def on_cursor_changed(self, widget):
        self.on_treeview_cursor_changed(widget)
        if self.selected_item is not None:
            self.selected_comic, model, selection_iter = self.selected_item
            logger.debug("comic selected " + str(self.selected_comic))
            pubsub.publish("comic.selected", self.selected_comic)
            #name = widget.get_model().get_value(iter,2)
        else:
            pubsub.publish("comic.deselected")

       
