#!/usr/bin/env python

import pubsub
import logging
import config
import ConfigParser
from random import choice
from bisect import bisect

logger = logging.getLogger(__name__)

class blammodel(object):
    
    def __init__(self, store):
        self.Store = store
        
        
    def LoadIni(self, data):
        for comic in data:
            try:
                c = self.Comics.Create(comic['name'])
                logger.info("Comic " + comic['name'] + " created from ini File.")
            except AlreadyExistsException, e:
                logger.info("Comic " + comic['name'] + " already in db. Not overwritten.")
        
    def createComic(self, name):
        try:
            return self.Comics.Create(name)
        except AlreadyExistsException, e:
            logger.info("Comic " + name + " already present.")
        
    def removeComic(self, comic):
        return self.Comics.Remove(comic)
        
    def createTag(self, name):
        try:
            tag = self.Tags.Create(name)
            return tag
        except AlreadyExistsException, e:
            logger.info("Tag " + name + " already present.")
        
    def removeTag(self, tag):
        pubsub.publish("Tag.removed", tag)
        removed = self.Tags.Remove(tag)
        return removed
        
    def createStrip(self, name):
        try:
            return self.Strips.Create(name)
        except AlreadyExistsException, e:
            logger.info("Strip " + name + " already present.")
            return None
        
    def removeStrip(self, strip):
        return self.Strips.Remove(strip)
        
    def createImage(self, name):
        try:
            return self.Images.Create(name)
        except AlreadyExistsException, e:
            logger.info("Image " + name + " already present.")
        
    def getTagByID(self, id):
        return self.Tags.getById(id)
        
    def getComicsForTag(self, tag):
        return tag._comics
        
    def Save(self):
        self.Store.Save()
        
    def getComicConfig(self, comic):
        return self.iniParser.get_config(comic)
        
    def details(self):
        result = "- - - - - - - - - -\n"
        for col in [self.Comics, self.Tags, self.Strips, self.Images]:
            for item in col:
                result += item.toString()
                result += "\n"
        result += "- - - - - - - - - -"
        return result
        
class blamobject(object):
    
    def getName(self):
        return self._name
        
    def setName(self, name):
        self._name = name
        
    Name = property(getName, setName)
    
    def getID(self):
        
        return self._ID
        
    def setID(self, id):
        self._ID = id
        
    ID = property(getID, setID)
        
class Comic(blamobject):
    
    def __init__(self, store, id, name):
        self.Store = store
        self._ID = id
        self._name = name
        self._strips = []
        self._tags = []
        self._lastURL = None
        pubsub.publish("Comic.created", name, self)
        
    def setLastURL(self, url):
        self._lastURL = url
        pubsub.publish("Comic.lastURL.added", self, url)
        
    def addTag(self, tag):
        for t in self._tags:
            if t.ID == tag.ID:
                logger.error(str(tag) + " already present in " + str(self))
                return
        self._tags.append(tag)
        tag._comics.append(self)
        pubsub.publish("Comic.Tag.added", tag, self)
    
    def removeTag(self, tag):
        for t in self._tags:
            if t.ID == tag.ID:
                self._tags.remove(tag)
                tag._comics.remove(self)
                pubsub.publish("Comic.Tag.removed", tag, self)
        
    def addStrip(self, strip):
        for s in self._strips:
            if s.ID == strip.ID:
                logger.error(str(strip) + " already present in " + str(self))
                return
        self._strips.insert(bisect(self._strips, strip), strip)
        pubsub.publish("Strip.added", strip, self)
    
    def getLast(self):
        try:
            return self._strips[-1]
        except:
            return None
    
    def getFirst(self):
        try:
            return self._strips[0]
        except:
            return None
    
    def getNext(self, last_strip):
        try:
            return self._strips[self._strips.index(last_strip)+ 1]
        except:
            return None
        
    def getPrevious(self, last_strip):
        try:
            return self._strips[self._strips.index(last_strip)- 1]
        except:
            return None
    
    def getRandom(self):
        try:
            return choice(self._strips) 
        except:
            return None
    
    def __repr__(self):
        return "Comic<" + self._name + ">"
    
    def __str__(self):
        return str(self._name)
        
    def toString(self):
        return "Comic<ID " + str(self._ID) + "|Name " + self._name + "|Tags " + str(self._tags) + "|Strips " + str(self._strips) + ">"
        
class Tag(blamobject):
    
    def __init__(self, store, id, name):
        self.Store = store
        self._ID = id
        self._name = name
        self._comics = []
        pubsub.publish("Tag.created", name, self)
        
    def __repr__(self):
        return "Tag<" + self._name + ">"
        
    def toString(self):
        return "Tag<ID " + str(self._ID) + "|Name " + self._name + "|Comics " + str(self._comics) + ">"
        
class Strip(blamobject):
    
    def __init__(self, store, id, name):
        self.Store = store
        self._ID = id
        self._name = name
        self._comic = None
        self._number = -1
        self._images = []
        pubsub.publish("Strip.created", name, self)
        
    def getNumber(self):
        return self._number
        
    def setNumber(self, number):
        self._number = number
        pubsub.publish("Strip.updated.number", number, self)
        
    Number = property(getNumber, setNumber)
    
    def getComic(self):
        return self._comic
        
    def setComic(self, comic):
        self._comic = comic
        comic.addStrip(self)
        pubsub.publish("Strip.updated.comic", comic, self)
        
    Comic = property(getComic, setComic)
    
    def __repr__(self):
        return "Strip<" + self._name + ">"
        
    def __cmp__(self, other):
        return self.Number - other.Number
    
    def toString(self):
        return "Strip<ID " + str(self._ID) + "|Name " + self._name + "|Comic " + str(self._comic) + "|Number " + str(self._number) + ">"
        
class Image(blamobject):
    
    def __init__(self, store, id, name):
        self.Store = store
        self._ID = id
        self._name = name
        self._path = None
        self._data = None
        self._strip = None
        self._url = None
        self._date = None
        pubsub.publish("Image.created", name, self)
        
    def getURL(self):
        return self._url
    
    def setURL(self, url):
        self._url = url
        pubsub.publish("Image.updated.URL", url, self)
    
    URL = property(getURL, setURL)
    
    def getDate(self):
        return self._date
    
    def setDate(self, date):
        self._date = date
        pubsub.publish("Image.updated.date", date, self)
    
    Date = property(getDate, setDate)
        
    def getPath(self):
        return self._path
        
    def setPath(self, path):
        self._path = path
        pubsub.publish("Image.updated.path", path, self)
        
    Path = property(getPath, setPath)
    
    def getData(self):
        return self._data
        
    def setData(self, data):
        self._data = data
        # no publish, as data does not get written into db
        # maybe publish so the gui knows...
        
    Data = property(getData, setData)
    
    def setStrip(self, strip):
        self._strip = strip
        pubsub.publish("Image.updated.strip", strip, self)
        
    def getStrip(self):
        return self._strip
        
    Strip = property(getStrip, setStrip)
    
    def __repr__(self):
        return "Image<"+self._name+">"
    
    def toString(self):
        return "Image<ID " + str(self._ID) + "|Name " + self._name + "|Strip " + str(self._strip) + "|Path " + str(self._path) +">"
        
class modelList(list):
    
    def __init__(self, store, items):
        list.__init__(self, items)
        for item in items:
            item.Parent = self
        self.Store = store
        self._create = lambda x: self._myRaise(self.__class__.__name__ + "._create")
        
    def _myRaise(self,message):
        raise NotImplementedException(message)
        
    def setType(self, type):
        self._type = type
        
    def getType(self):
        return self._type
        
    type = property(getType, setType)
    
    def namePresent(self, name):
        for n in [i.Name for i in self]:
            if n == name:
                return True
        return False
        
    def Index(self, item):
        for i, it in enumerate(self):
            if it == item:
                return i
        return -1
        
    def Create(self, name):
        if self.namePresent(name):
            raise AlreadyExistsException(self._type, name)
        item = self._create(name)
        item.Parent = self
        self.append(item)
        return item
        
    def Remove(self, item):
        index = self.Index(item)
        if index == -1:
            raise Exception(str(item) + " not present in " + str(self))
        item = self.pop(index)
        return item
        
    def getById(self, id):
        for item in self:
            if item.ID == id:
                return item
        return None
        
    def __eq__(self, other):
        if len(self) != len(other):
            return False
        for left, right in zip(self, other):
            if not left == right:
                return False
        return True
        
class ComicList(modelList):
    pass
    
class TagList(modelList):
    pass
    
class StripList(modelList):
    pass
    
class ImageList(modelList):
    pass
    
class IniParser(object):
    def __init__(self, path):
        self.Path = path
        self.cp = ConfigParser.ConfigParser()
        self.cp.readfp(open(self.Path))
        self.read()
        
    def read(self):
        self.data = []
        for section in self.cp.sections():
            if not section == "Comics":
                current = {}
                current['name'] = section
                for (key, value) in self.cp.items(section):      
                    current[key] = value
                self.data.append(current)
    
    def get_config(self, comic):
        data = {}
        for (key, value) in self.cp.items(comic.Name):
            data['name'] = comic.Name
            data[key] = value
        return data
        
        
class NotImplementedException(Exception):
    
    def __init__(self, function):
        self.function = function

    def __str__(self):
        return "Unimplemented function in '%s'."%self.function
        
class AlreadyExistsException(Exception):
    
    def __init__(self, type, name):
        self.name = name
        self.type = type

    def __str__(self):
        return "Object " + str(type) + " with name " + name + " already exists."
        
class NoNameException(Exception):
    
    def __init__(self, type, name):
        self.name = name
        self.type = type

    def __str__(self):
        return "Object " + str(type) + " with name " + name + " does not exists."
