# -*- coding: utf-8 -*-
from sqlite3 import dbapi2 as sqlite
import sqlite3
import pubsub
import blamobjects
import logging
import os

logger = logging.getLogger(__name__)

class BlamStore(object):
    
    def __init__(self, path, autoSave = True):
        self.Path = path
        self.AutoSave = autoSave
        self.Dirty = False
        self.Version = 1
        existing = True
        #print "mach was!"
        if not os.path.exists(self.Path):
            #print "mach ich ja"
            logger.debug('Initializing: '+ self.Path)
            connection = self.Initialize()
            existing = False
        else:
            connection = self.getConnection()
        self.dbconn = connection
        self.Dirty = True
        self.commitIfAppropriate()
        self.Meta = self.GetMeta()
        if self.Meta['VERSION'] < self.Version:
            self.upgradeDB(self.Meta['VERSION'])
        self.Subscriptions = (
            (self.onComicTagAdded, "Comic.Tag.added"),
            (self.onComicTagRemoved, "Comic.Tag.removed"),
            (self.onTagRemoved, "Tag.removed"),
            (self.removeTagComicLinks, "Tag.removed"),
            (self.onStripNumber, "Strip.updated.number"),
            (self.onStripComic, "Strip.updated.comic"),
            (self.onImagePath, "Image.updated.path"),
            (self.onImageStrip, "Image.updated.strip"),
            (self.onComicLastUrlSet, "Comic.lastURL.added"),
            (self.onImageURL,"Image.updated.URL"),
            (self.onImageDate,"Image.updated.date"),
            (self.onExit, "exiting"),
        )
        for callback, topic in self.Subscriptions:
            pubsub.subscribe(topic, callback)
            
    def saveCursor(self):
        try:
            c = self.dbconn.cursor()
        except sqlite3.ProgrammingError:
            c = self.renewConnection().cursor()
        return c
            
    def getConnection(self):
        logger.debug('Loading: '+ self.Path)
        connection = sqlite.connect(self.Path)
        return connection
        
    def renewConnection(self):
        con = self.getConnection()
        self.dbconn = con
        return con
            
    def getModel(self):
        return self.blamModel

    def CreateModel(self):
        logger.debug('Creating model...')
        self.blamModel = blamobjects.blammodel(self)
        return self.blamModel
        
    def CreateComic(self, name):
        cursor = self.saveCursor()
        cursor.execute('INSERT INTO comics VALUES (null, ?, null)', (name,))
        ID = cursor.lastrowid
        self.Dirty = True
        self.commitIfAppropriate()
        comic = blamobjects.Comic(self,ID,name)
        return comic
        
    def CreateTag(self, name):
        cursor = self.saveCursor()
        cursor.execute('INSERT INTO tags VALUES (null, ?)', (name,))
        ID = cursor.lastrowid
        self.Dirty = True
        self.commitIfAppropriate()
        tag = blamobjects.Tag(self,ID,name)
        return tag
        
    def CreateStrip(self, name):
        cursor = self.saveCursor()
        cursor.execute('INSERT INTO strips VALUES (null, null, null, ?)', (name,))
        ID = cursor.lastrowid
        self.Dirty = True
        self.commitIfAppropriate()
        strip = blamobjects.Strip(self,ID,name)
        return strip
        
    def CreateImage(self, name):
        cursor = self.saveCursor()
        cursor.execute('INSERT INTO images VALUES (null, null, null, ?, null, null)', (name,))
        ID = cursor.lastrowid
        self.Dirty = True
        self.commitIfAppropriate()
        image = blamobjects.Image(self,ID,name)
        return image
        
    def Initialize(self):
        connection = sqlite.connect(self.Path)
        cursor = connection.cursor()
        cursor.execute('CREATE TABLE comics (id INTEGER PRIMARY KEY, name VARCHAR(255), last_url VARCHAR(255))')
        cursor.execute('CREATE TABLE tags (id INTEGER PRIMARY KEY, name VARCHAR(255))')
        cursor.execute('CREATE TABLE strips (id INTEGER PRIMARY KEY, comicId INTEGER, number FLOAT, name VARCHAR(255))')
        cursor.execute('CREATE TABLE images (id INTEGER PRIMARY KEY, stripId INTEGER, path VARCHAR(255), name VARCHAR(255), url VARCHAR(255), date DATE)')
        cursor.execute('CREATE TABLE tags_comics_link (id INTEGER PRIMARY KEY, comicId INTEGER, tagId INTEGER)')
        cursor.execute('CREATE TABLE meta (id INTEGER PRIMARY KEY, name VARCHAR(255), value VARCHAR(255))')
        cursor.execute('INSERT INTO meta VALUES (null, ?, ?)', ('VERSION', self.Version))
        return connection
        
    def GetMeta(self):
        try:
            results = self.saveCursor().execute('SELECT * FROM meta').fetchall()
        except sqlite3.OperationalError:
            meta = {'VERSION': 0}
        else:
            meta = {}
            for uid, key, value in results:
                # All values come in as strings but some we want to cast.
                if key == "VERSION":
                    value = int(value)
                meta[key] = value
        return meta
        
    def upgradeDB(self, oldVersion):
        # Serious stuff to do here, for now:
        logger.error( "HAT HIER WIEDER JEMAND VERGESSEN DIE DB ZU LÖSCHEN??")
            
    def _get(self, create, table, con=None):
        if not con:
            c = self.saveCursor()
        else:
            c = con.cursor()
        return [create(result) for result in c.execute("SELECT * FROM " + table).fetchall()] 
        
    def getComics(self, con = None):
       return self._get(self.result2comic, "comics", con)
       
    def getTags(self, con = None):
       return self._get(self.result2tag, "tags", con)
       
    def getStrips(self, con = None):
       return self._get(self.result2strip, "strips", con)
       
    def getImages(self, con = None):
       return self._get(self.result2image, "images", con)
       
    def result2comic(self, result):
        ID, name, lastURL = result
        comic = blamobjects.Comic(self, ID, name)
        comic._lastURL = lastURL
        return comic
        
    def result2tag(self, result):
        ID, name = result
        return blamobjects.Tag(self, ID, name)
        
    def result2strip(self, result):
        ID, comicID, number, name = result
        strip = blamobjects.Strip(self, ID, name)
        if number:
            strip._number = number
        return strip
        
    def result2image(self, result):
        ID, stripID, path, name, url, date = result
        image = blamobjects.Image(self, ID, name)
        if path:
            image._path = path
        if url:
            image._url = url
        if date: 
            image._date = date
        return image
        
    def onStripNumber(self, number, strip):
        logger.debug("Setting number of " +str(strip) + " to " + str(number))
        self.saveCursor().execute('UPDATE strips SET number=? WHERE id=?', (number, strip.ID))
        self.Dirty = True
        self.commitIfAppropriate()
        
    def onStripComic(self, comic, strip):
        logger.debug("Setting Comic of " +str(strip) + " to " + str(comic))
        self.saveCursor().execute('UPDATE strips SET comicId=? WHERE id=?', (comic.ID, strip.ID))
        self.Dirty = True
        self.commitIfAppropriate()
        
    def onComicTagAdded(self, tag, comic):
        logger.debug("Adding " + str(tag) + " to " + str(comic))
        cursor = self.saveCursor()
        cursor.execute('INSERT INTO tags_comics_link VALUES (null, ?, ?)', (comic.ID, tag.ID,))
        self.Dirty = True
        self.commitIfAppropriate()
        
    def onComicLastUrlSet(self, comic, url):
        logger.debug("Setting LastURL of " + str(comic) + " to " + url)
        cursor = self.saveCursor()
        cursor.execute('UPDATE comics SET last_url = ? WHERE id = ?', (url,comic.ID,))
        self.Dirty = True
        self.commitIfAppropriate()
    
    def onComicTagRemoved(self, tag, comic):
        logger.debug("Removing " + str(tag) + " to " + str(comic))
        cursor = self.saveCursor()
        cursor.execute('DELETE FROM tags_comics_link WHERE tagID = ? AND comicID = ?', (tag.ID, comic.ID))
        self.Dirty = True
        self.commitIfAppropriate()
      
    def onTagRemoved(self, tag):
        logger.debug("Removing " + str(tag))
        cursor = self.saveCursor()
        cursor.execute('DELETE FROM tags WHERE id=?', (tag.ID,))
        self.Dirty = True
        self.commitIfAppropriate()
        
    def removeTagComicLinks(self, tag):
        logger.debug("Removing Links of Tag " + str(tag))
        cursor = self.saveCursor()
        cursor.execute('DELETE FROM tags_comics_link WHERE tagId=?', (tag.ID,))
        self.Dirty = True
        self.commitIfAppropriate()
        
    def onImagePath(self, path, image):
        logger.debug("Setting Path of " + str(image) + " to " + str(path))
        cursor = self.saveCursor()
        cursor.execute('UPDATE images SET path=? WHERE id=?', (path, image.ID))
        self.Dirty = True
        self.commitIfAppropriate()
        
    def onImageURL(self, url, image):
        logger.debug("Setting URL of " + str(image) + " to " + str(url))
        cursor = self.saveCursor()
        cursor.execute('UPDATE images SET url=? WHERE id=?', (url, image.ID))
        self.Dirty = True
        self.commitIfAppropriate()
        
    def onImageDate(self, date, image):
        logger.debug("Setting Date of " + str(image) + " to " + str(date))
        cursor = self.saveCursor()
        cursor.execute('UPDATE images SET date=? WHERE id=?', (date, image.ID))
        self.Dirty = True
        self.commitIfAppropriate()
        
    def onImageStrip(self, strip, image):
        logger.debug("Setting Strip of " + str(image) + " to " + str(strip))
        if not image in strip._images:
            strip._images.append(image)
        cursor = self.saveCursor()
        cursor.execute('UPDATE images SET stripId=? WHERE id=?', (strip.ID, image.ID))
        self.Dirty = True
        self.commitIfAppropriate()
        
    def onExit(self):
        logger.debug("Exiting")        
        
    def Save(self):
        import time; t = time.time()
        self.saveCursor()
        self.dbconn.commit()
        logger.debug("Committed in %s seconds" % (time.time()-t))
        self.Dirty = False
        
    def Close(self):
        self.dbconn.close()
        for callback, topic in self.Subscriptions:
            unsubscribe(topic, callback)
    
    def commitIfAppropriate(self):
        if self.AutoSave and self.Dirty:
            self.Save()
            
    def onExit(self, message):
        if self.Dirty:
            pubsub.publish("warning.dirty exit", message.data)

    def __del__(self):
        self.commitIfAppropriate()
        self.Close()
