from threading import Thread
import logging, gtk
import config
import blamobjects, threadpool

logger = logging.getLogger(__name__)

class MyThreadQueue(Thread):
    
    def __init__(self):
        Thread.__init__(self)
        self.workers = []
        
    def put(self, worker):
        self.workers.append(worker)
        
    def run(self):
        while len(self.workers) > 0:
            current = self.workers.pop(0)
            logger.info("Starting Thread: " + str(current))
            current.start()
            current.join()
            logger.info("Thread finished: " + str(current))
            

class ObjectDresser(Thread):
    
    def __init__(self, session):
        Thread.__init__(self)
        self.session = session
        
    def run(self):
        logger.info("Start Linking of Objects")
        
        con = self.session['store'].getConnection()
        comics = self.session['model'].Comics
        strips = self.session['model'].Strips
        tags = self.session['model'].Tags
        images = self.session['model'].Images
        taglinks = con.cursor().execute("SELECT * FROM tags_comics_link").fetchall()
        imagelinks = con.cursor().execute("SELECT * FROM images").fetchall()
        striplinks = con.cursor().execute("SELECT * FROM strips").fetchall()
        
        monitor = self.session["progressManager"].add_monitor(5, "Loading Strips", gtk.STOCK_CONNECT)
        monitor.amount = len(taglinks) + len(striplinks) + len(imagelinks)

        for id, comicID, tagID in taglinks:
            comic = comics.getById(comicID)
            tag = tags.getById(tagID)
            comic._tags.append(tag)
            tag._comics.append(comic)
            monitor.progress_update_ammount(1)
        
        for id, comicID, number, name in striplinks:
            if comicID:
                comic = comics.getById(comicID)
                strip = strips.getById(id)
                comic._strips.append(strip)
                strip._comic = comic
            monitor.progress_update_ammount(1)
       
        for comic in comics:
            comic._strips.sort()
        
        for ID, stripID, path, name, url, date in imagelinks:
            if stripID:
                image = images.getById(ID)
                strip = strips.getById(stripID)
                image._strip = strip
                strip._images.append(image)
            monitor.progress_update_ammount(1)                
     
        self.session["progressManager"].remove_monitor(5)
        logger.info("Finished Linking of Objects")
        
    def __repr__(self):
        return "ObjectDresser Thread"
        
class ModelLoader(Thread):
    
    def __init__(self, session):
        Thread.__init__(self)
        self.session = session
        
    def run(self):
        logger.info("Start Loading of Objects")
        monitor = self.session["progressManager"].add_monitor(7, "Loading Comics", gtk.STOCK_INFO)

        store = self.session['store']
        con = store.renewConnection()
        model = self.session['model']
        model.Comics = blamobjects.ComicList(store, store.getComics(con=con))
        model.Comics.setType(blamobjects.Comic)
        model.Comics._create = store.CreateComic
        monitor.progress_update(20)
        model.Tags = blamobjects.TagList(store, store.getTags(con=con))
        model.Tags.setType(blamobjects.Tag)
        model.Tags._create = store.CreateTag
        monitor.progress_update(40)
        model.Strips = blamobjects.StripList(store, store.getStrips(con=con))
        model.Strips.setType(blamobjects.Strip)
        model.Strips._create = store.CreateStrip
        monitor.progress_update(60)
        model.Images = blamobjects.ImageList(store, store.getImages(con=con))
        model.Images.setType(blamobjects.Image)
        model.Images._create = store.CreateImage
        monitor.progress_update(80)
        model.iniParser = blamobjects.IniParser(config.INI_PATH)
        model.LoadIni(model.iniParser.data)
        monitor.progress_update(100)
        self.session['mainWindow'].reload_from_data()
        
        self.session["progressManager"].remove_monitor(7)
        logger.info("Finish Loading of Objects")
        
    def __repr__(self):
        return "ModelLoader Thread"
        return "ModelLoader Thread"
