from config import *
import favicons
import http
import locale, gtk
import threadpool
import session
import pubsub
import time
import datetime

logger = logging.getLogger(__name__)


def _checkConf(conf, needed):
    for need in needed:
        if not need in conf:
            logger.error("Needed Configuration " + str(need) + " not present in " + str(conf))
            return False
    return True

    
class ComicThread():
    def __init__(self):
        pubsub.subscribe('progress.cancel', self.progress_canceled)
        self.session = session.get_session()
        self.pools = {}
        self.downloading = {}

    

    def loadComic(self, comic):
        conf = self.session['model'].getComicConfig(comic)
        if NAVTYPE in conf:
            type = conf[NAVTYPE]
        else:
            logger.error("No navtype in conf: " + str(conf))
            return
        if type == NUMBER:
            self._loadComicNumber(comic, conf)
        elif type == DATEINPAGE:
            
            def _callback(args): 
                logger.info("DateInPage Download of "+str(args[1]['name'])+" finished.")
                del pool
            
            pool = threadpool.ThreadPool(1)
            pool.queueTask(self._loadComicDateInPage, args = (comic, conf, pool), taskCallback = lambda args: _callback(args))
        elif type == ARCHIV:
            self._loadComicArchive(comic, conf)
        else:
            logger.error("Unimplemented Navigation Type: " + str(type))
            
    def _loadComicArchive(self, comic, conf):
        
        def _make_callback(comic, number, url):
            return lambda x: self._process_download((comic, number, url), x, "Amount")
        
        pool = self.pools[comic.ID] = threadpool.ThreadPool(5)
        self.downloading[comic.ID] = 1
        logger.debug("Trying to download archiv navigation comic " + conf['name'])
        monitor = self.session["progressManager"].add_monitor(comic._ID, comic.Name, gtk.STOCK_GO_DOWN)
        if not _checkConf(conf, [ARCHIVEITEM, ARCHIVEURL, IMGEXP]):
            return None
        conf[HOMEPAGE] = conf[ARCHIVEURL]
        urls = http.getArchiveURLS(conf)
        comicUrls = []
        #comicDates = []
        imageUrls = []
        for tuple in urls[ARCHIVEITEM]:
            #print tuple
            comicUrls.append(tuple[0])
            #year = tuple[3]
            #month = tuple[1]
            #day = tuple[2]
            #year = locale.atoi(year)
            #month = locale.atoi(month)
            #day = max(locale.atoi(day), 1)
            #print year, month, day
            #comicDates.append(time.mktime(datetime.date(year,month,day).timetuple()))
        #print comicUrls
        #print comicDates
        monitor.amount = len(comicUrls)
        for i in range(0,len(comicUrls)):
            currentC = comicUrls[i]
            #currentD = comicDates[i]
            pool.queueTask(http.download_image_from_URL, args = [currentC, conf[IMGEXP]], taskCallback = _make_callback(comic, i, currentC))
        
            
    def _loadComicDateInPage(self, args):
        
        comic, conf, pool = args
        
        def _make_callback(comic, number, url):
            return lambda x: self._process_download((comic, number, url), x, "Text")
        
        pool = self.pools[comic.ID] = threadpool.ThreadPool(5)
        self.downloading[comic.ID] = 1
        logger.debug("Trying to download dateInPage navigation comic " + conf['name'])
        monitor = self.session["progressManager"].add_monitor(comic._ID, comic.Name, gtk.STOCK_GO_DOWN)
        if not _checkConf(conf, [HOMEPAGE, DATEREGEX, PREVEX, IMGEXP, BASE, RECURSION_BASE, FIRSTURL]):
            return None
        number = 1
        notAtFirst = True
        while notAtFirst:
            #print "current config: ", conf
            urls = http.getDateInPageURLS(conf)
            url = urls[IMGEXP][0]
            #print "URL!!",url
            pool.queueTask(http.download_image, args = url, taskCallback = _make_callback(comic, number, url))
            if not conf[FIRSTURL] == conf[HOMEPAGE]:
                number += 1
                #print "urls", urls
                newHome = str(conf[RECURSION_BASE]) + urls[PREVEX].pop()
                #print "new Home. From: ",conf[HOMEPAGE],"to ", newHome
                conf[HOMEPAGE] = newHome
                logger.debug("At iteration " + str(number) + " of " + str(comic))
            else:
                logger.info("Reached first Comic of " + str(comic))
                notAtFirst = False
        pool.joinAll(True, True)
        del pool
        del self.downloading[comic.ID]
        


    def _loadComicNumber(self, comic, conf):
        
        def _make_callback(comic, number, url):
            return lambda x: self._process_download((comic, number, url), x, "Amount")
        
        pool = self.pools[comic._ID] = threadpool.ThreadPool(5)
        self.downloading[comic._ID] = 1
        
        logger.debug("Trying to download number navigation comic " + conf['name'])
        monitor = self.session["progressManager"].add_monitor(comic._ID, comic.Name, gtk.STOCK_GO_DOWN)
        if not _checkConf(conf, [FIRSTNUMBER, NUMBERREGEX, IMGURL, HOMEPAGE, MINDIGITS]):
            return None
        urls = http.getNumberURLS(conf)
        downloads = self._make_number_downloads(conf, urls)
        monitor.amount = len(downloads)
        for number, url in downloads:
            pool.queueTask(http.download_image, args = url, taskCallback = _make_callback(comic, number, url))

    def terminate_all(self):
        for key, pool in self.pools.items():
            pool.joinAll(False, False)
            del pool
        self.downloading ={}
        self.pools = {}

    def progress_canceled(self, id):
        self.pools[id].joinAll(False, False)
        del self.downloading[id]
        del self.pools[id]

    def _process_download(self, data, response, update = "Amount"): 
        comic, number, url = data
        strip = self._make_strip(comic, number, url)
        if response and strip:
            fileType = self._get_filetype_from_response_info(response.info())
            f = self._make_file_for_strip(comic, number, fileType)
            image = self._make_image_for_strip(strip, f)
            if image:
                image.URL = url
                image.Date = time.time()
                f.write(response.read())
                f.close()
                logger.debug('Saved image ' + str(number) + " of comic " + str(comic))
                comic.setLastURL(url)
            else:
                logger.info('Image for strip ' + str(strip) + " already present")
        else:
            logger.debug('No response for image ' + str(number) + " of comic " + str(comic))
        if comic._ID in self.downloading:
            if update == "Amount":
                self.session["progressManager"].monitors[comic._ID].progress_update_ammount(1)
            elif update == "Text":
                self.session["progressManager"].monitors[comic._ID].progressText = "#" + str(number)
                self.session["progressManager"].monitors[comic._ID].progress_update_text()
     
            
    def _make_image_for_strip(self, strip, file):
        image = self.session['model'].createImage(self._make_image_name(strip))
        if image:
            image.Strip = strip
            image.Path = file.name
            return image
        return None
            
    def _make_strip(self, comic, number, url):
        strip = self.session['model'].createStrip(self._make_strip_name(comic, number))
        if strip:
            strip.Number = number
            strip.Comic = comic
            return strip
        return None
        
    def _make_strip_name(self, comic, number):
        return comic.Name + "#" + str(number)
        
    def _make_image_name(self, strip):
        return strip.Name + "_image"
            
    def _make_file_for_strip(self, comic, number, fileType):
        #print "making file for Strip number ", number, " of comic ", comic
        baseDir = get_comic_folder(comic)
        filePath = baseDir + str(number) + "." + fileType
        f = open(filePath, "w")
        return f
            
    def _make_number_downloads(self, conf, urls):
        first = locale.atoi(conf[FIRSTNUMBER])
        print urls[NUMBERREGEX]
        last = locale.atoi(urls[NUMBERREGEX][0])
        min = locale.atoi(conf[MINDIGITS])
        #print type(last)
        #print first, last
        #print comic.toString()
        result = []
        for i in range(first,last+1):
            if min > 0:
                rep = self._format(i,min)
            else:
                rep = str(i)
            result.append((i,conf[IMGURL].replace('$n',rep)))
        return result
            
    def _format(self, number, min):
        numberlength = 1
        current = number
        while current/10 >= 1:
            numberlength += 1
            current /= 10
        if numberlength >= min:
            return str(number)
        newlength = min - numberlength
        pre = ""
        for i in range(0,newlength):
            pre += "0"
        return pre+str(number)
            
    DELIMITER = 'Content-Type:'
            
    def _get_filetype_from_response_info(self, info):
        #print info.getmaintype()
        subtype = info.getsubtype()
        return subtype

