import os, logging
import http, session
from config import *
import threadpool, pubsub

logger = logging.getLogger(__name__)


def _checkConf(conf, needed):
    for need in needed:
        if not need in conf:
            logger.error("Needed Configuration " + str(need) + " not present in " + str(conf))
            return False
    return True


class FaviconThread():
    def __init__(self):
        #pubsub.subscribe('progress.cancel', self.progress_canceled)
        self.session = session.get_session()
        self.pool = threadpool.ThreadPool(5)

    def download_favicon(self, comic):
        conf = self.session['model'].getComicConfig(comic)
        if not _checkConf(conf, [FAVICON]):
            return False
        logger.debug("Getting favicon from " + conf[FAVICON] + " ...")
        favicon = http.download_image(conf[FAVICON])
        f = FAVICONS_DIR + str(comic._ID)
        if favicon:
            file = open(f, "w")
            file.write(favicon.read())
            file.close()
            logger.debug("Favicon: " + conf[FAVICON] +" downloaded and saved.")
            return True    
        return False
 
    def _process_download(self, comic, response):
        if response:
            pubsub.publish("favicon.downloaded", comic)
 
 
    def get_favicon(self, comic):
        def get_callback():
            return lambda x: self._process_download(comic, x)
        
        f = FAVICONS_DIR + str(comic.ID)
        if not os.path.isfile(f): 
            self.pool.queueTask(self.download_favicon, args = comic, taskCallback = get_callback() )
            return DATA_DIR+"blam_icon_64_64.png"
        return f

