import gtk
import pubsub
import logging
import session

logger = logging.getLogger(__name__)

class ImageView(gtk.Image):
    def __init__(self):
        gtk.Image.__init__(self)
        self.session = session.get_session()
        
        self.current_comic = None
        self.current_strip = None
        self.current_pixbuf = None
        
        pubsub.subscribe('comic.selected', self.show_comic)
        pubsub.subscribe('comic.deselect', self.deselect_comic)
        pubsub.subscribe('comic.last', self.show_last)
        pubsub.subscribe('comic.first', self.show_first)
        pubsub.subscribe('comic.previous', self.show_previous)
        pubsub.subscribe('comic.next', self.show_next)
        pubsub.subscribe('comic.random', self.show_random)
        pubsub.subscribe('comic.zoom_in', self.zoom_in)
        pubsub.subscribe('comic.zoom_out', self.zoom_out)
        
    def deselect_comic(self):
        self.current_comic = None
        self.clear()

    def show_comic(self, comic):
        self.current_comic = comic
        self.show_last()
    
    def show_last(self):
        if self.current_comic:
            self.current_strip = self.current_comic.getLast()
            self.__show_strip(self.current_strip)
            
    def show_first(self):
        if self.current_comic:
            self.current_strip = self.current_comic.getFirst()
            self.__show_strip(self.current_strip)

    def show_previous(self):
        if self.current_comic:
            self.current_strip = self.current_comic.getPrevious(self.current_strip)
            self.__show_strip(self.current_strip)

    def show_next(self):
        if self.current_comic:
            self.current_strip = self.current_comic.getNext(self.current_strip)
            self.__show_strip(self.current_strip)
            
    def show_random(self):
        if self.current_comic:
            self.current_strip = self.current_comic.getRandom()
            self.__show_strip(self.current_strip)

    def __show_strip(self, strip):
        if strip is not None:
            self.current_pixbuf = gtk.gdk.pixbuf_new_from_file(strip._images[0].Path)
            self.__show_pixbuf()
    
    def __show_pixbuf(self):
        self.set_from_pixbuf(self.current_pixbuf)
        self.scroll_to_top()
        
    def scroll_to_top(self):
        self.session['imageviewScroll'].get_vadjustment().set_value(0)
        self.session['imageviewScroll'].get_hadjustment().set_value(0)
        
    def zoom_in(self):
        pass
        
    def zoom_out(self):
        pass

    
