#!/bin/env python
# -*- coding: utf8 -*-

# Bunch of meta data, used at least in the about dialog

__appname__ = 'blam!'
__version__ = '0.1'
__description__ = 'A program to easily read online comics'
__url__='https://launchpad.net/blam'
__authors__ = ['Bastian Kennel (bastian.kennel(at)gmail.com)', 'Wolfgang Steitz (wsteitz(at)gmail.com)']
__copyright__ = '''\
Copyright (c) 2008-2009 Wolfgang Steitz, Bastian Kennel

'''
__license__='''\
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
'''


try:
    import pygtk
    pygtk.require("2.0")
except:
    raise Exception("PyGTK Version >=2.0 required")


import sys
import gtk
import logging
import os
import gobject
import treeviews, context_menus, config, blamobjects, pubsub, menubar, imageview, blamstore, blamthreads
import toolbars, progress, comicthread, favicons

import session

logger = logging.getLogger(__name__)


class MainWindow(gtk.Window):
    
    def __init__(self):
        # Create the toplevel window
        gtk.Window.__init__(self)
        self.connect("destroy", self.quit)
        self.connect('key-press-event',self.catch_button)
        self.set_title(__appname__)
        self.set_icon_from_file(config.DATA_DIR+'blam.png')
        self.session = session.get_session()

        # Use two thirds of the screen by default
        screen = self.get_screen()
        monitor = screen.get_monitor_geometry(0)
        width = int(monitor.width * 0.66)
        height = int(monitor.height * 0.66)
        self.set_default_size(width, height)

        vbox = gtk.VBox()
        self.add(vbox)
        
        main_tree = treeviews.MainTree()
        self.session['mainTree'] = main_tree
        
        vbox.pack_start(menubar.MenuBar(), expand=False, fill=False)
 
        hpaned = gtk.HPaned()
        hpaned.set_position(int(width*0.15))
        vbox.pack_start(hpaned)

        self.tag_vbox = gtk.VBox()
        hpaned.pack1(self.tag_vbox)
        self.tag_vbox.pack_start(toolbars.TagToolbar(), expand=False, fill=False)
        self.tag_vbox.pack_start(main_tree)
        
        progress_box = gtk.VBox()
        self.tag_vbox.pack_start(progress_box, expand = False)
        self.session['progressManager'] = progress.ProgressManager(progress_box)
        
        comic_vbox = gtk.VBox()
        hpaned.pack2(comic_vbox)
        vpaned = gtk.VPaned()
        vpaned.set_position(int(height*0.3))
        comic_vbox.pack_start(vpaned)

        scrolled_window1 = gtk.ScrolledWindow()
        scrolled_window1.set_policy(gtk.POLICY_AUTOMATIC,gtk.POLICY_AUTOMATIC)
        scrolled_window2 = gtk.ScrolledWindow()
        scrolled_window2.set_policy(gtk.POLICY_AUTOMATIC,gtk.POLICY_AUTOMATIC)
        vpaned.add1(scrolled_window1)
        vpaned.add2(scrolled_window2)
        
        
        comic_tree = treeviews.ComicTree()
        scrolled_window1.add(comic_tree)
        self.session['comicTree'] = comic_tree
        
        self.session['imageviewScroll'] = scrolled_window2
        scrolled_window2.add_with_viewport(imageview.ImageView())
        #Comic Toolbar
        comic_vbox.pack_start(toolbars.ComicToolbar() , expand=False, fill=False)

        statusbar = gtk.Statusbar()
        vbox.pack_start(statusbar, expand=False, fill=False)
        self.show_all()

    def reload_from_data(self):
        """Called when we want to reset everything based
        on internal data."""
        self.session['mainTree'].clear()
        self.session['comicTree'].clear()
        self.session['mainTree'].insert_categories()
        tags = self.session['model'].Tags
        for tag in tags:
            self.session['mainTree'].insert_tag(tag.Name, tag)
            #print tag.toString()
        self.session['mainTree'].expand_all()
        
    def catch_button(self, window, event):
        #keyval = event.keyval
        #name = gtk.gdk.keyval_name(keyval)
        mod = gtk.accelerator_get_label(event.keyval,event.state)
        if mod == 'Alt+Mod2+Left':
            pubsub.publish("comic.previous")
            return True
        elif mod == 'Alt+Mod2+Right':
            pubsub.publish("comic.next")
            return True
        elif mod == 'Mod2+R':
            pubsub.publish("comic.random")
            return True
        elif mod == 'Mod2+F':
            pubsub.publish("comic.first")
            return True
        elif mod == 'Mod2+L':
            pubsub.publish("comic.last")
            return True
    
    def quit(self, widget):
        self.session['comicThread'].terminate_all()
        gtk.main_quit(widget)
        

def main():
    config.create_folders()
    main_window = MainWindow()
    s = session.get_session()
    s['mainWindow']   = main_window
    s['store']        = blamstore.BlamStore(config.DB_PATH)
    s['store'].CreateModel()
    s['model']        = s['store'].blamModel
    s['threadQueue'] = blamthreads.MyThreadQueue()
    
    gobject.threads_init()
    loadthread = blamthreads.ModelLoader(s)
    s['threadQueue'].put(loadthread)
    linkthread = blamthreads.ObjectDresser(s)
    s['threadQueue'].put(linkthread)
    #Get the local path
    local_path = os.path.realpath(os.path.dirname(sys.argv[0]))
    s['threadQueue'].start()
    s['comicThread'] = comicthread.ComicThread()
    s['faviconThread'] = favicons.FaviconThread()

    gtk.main()    


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG, 
         format="%(levelname)s %(asctime)s %(funcName)s %(lineno)d %(message)s")
    main()
